"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reportImageInfo = exports.ImageInfoReporter = void 0;
const lodash_1 = __importDefault(require("lodash"));
const registries_1 = require("../api/registries");
const codefresh_1 = require("../api/codefresh");
const types_1 = require("./types");
class ImageInfoReporter {
    constructor(inputs, notify) {
        this.image = inputs.imageName;
        this.registryClient = new registries_1.RegistryClient(inputs);
        this.workflowName = inputs.workflow?.name;
        this.workflowUrl = inputs.workflow?.workflowUrl;
        this.logsUrl = inputs.workflow?.logsUrl;
        this.host = inputs.codefresh.host;
        this.apiKey = inputs.codefresh.apiKey;
        this.notify = notify;
        this.codefreshAPI = new codefresh_1.CodefreshAPI(this.host, this.apiKey);
    }
    async reportImageInfo() {
        console.log('starting image reporter library');
        const registry = await this.getRegistry();
        const manifest = await registry.getManifest();
        if (manifest.manifests) {
            const parentRepoDigest = manifest.config?.repoDigest;
            const reportResult = await Promise.all(lodash_1.default.map(manifest?.manifests, async (manifestConfig) => {
                if (manifestConfig.platform.os === 'unknown') {
                    return {};
                }
                const archManifest = await registry.getManifest(manifestConfig.digest, manifestConfig.mediaType);
                console.log(`image manifest has been fetched:`, '\n', archManifest);
                return await this.getConfigAndReportImageInfo(archManifest, parentRepoDigest);
            }));
            const parentRegistryResult = await this.codefreshAPI.createImageRegistry({
                imageRegistry: {
                    binaryId: reportResult[0].imageBinary.id,
                    imageName: this.image,
                    repoDigest: parentRepoDigest,
                    created: new Date(),
                    manifestList: true
                }
            });
            console.log('registryMutation payload:', parentRegistryResult);
        }
        else {
            console.log(`image manifest has been fetched:`, '\n', manifest);
            await this.getConfigAndReportImageInfo(manifest);
        }
        console.log('image reported successfully');
    }
    async getRegistry() {
        if (this.registry) {
            return this.registry;
        }
        const client = await this.registryClient.getRegistryClient(this.image);
        console.log(`using ${client.constructor.name} client`);
        const registry = client.repoTag(this.image);
        this.registry = registry;
        return registry;
    }
    async getConfigAndReportImageInfo(manifest, parentRepoDigest) {
        const registry = await this.getRegistry();
        const config = await registry.getConfig(manifest);
        console.log(`image config has been fetched:`, '\n', config);
        const repositoryName = lodash_1.default.get(this.registryClient.parseImageName(this.image), 'repository');
        const imageLink = `${this.host}/2.0/images/${encodeURIComponent(repositoryName)}/${manifest.config.digest}/${encodeURIComponent(this.image)}`;
        const size = this.registryClient.getRegistryImageSize(manifest);
        try {
            await this.notify(types_1.NOTIFICATIONS.IMAGE_NAME, this.image);
        }
        catch (error) {
            console.error(`Can't call notification "${types_1.NOTIFICATIONS.IMAGE_NAME}". ${error.message}`);
        }
        try {
            await this.notify(types_1.NOTIFICATIONS.IMAGE_SHA, lodash_1.default.get(manifest, 'config.digest'));
        }
        catch (error) {
            console.error(`Can't call notification "${types_1.NOTIFICATIONS.IMAGE_SHA}". ${error.message}`);
        }
        try {
            await this.notify(types_1.NOTIFICATIONS.IMAGE_LINK, imageLink);
        }
        catch (error) {
            console.error(`Can't call notification "${types_1.NOTIFICATIONS.IMAGE_LINK}". ${error.message}`);
        }
        const configCreated = config.created || new Date().toISOString();
        const imageBinaryVars = {
            imageBinary: {
                id: manifest.config.digest,
                created: configCreated,
                imageName: this.image,
                dockerFile: await this.registryClient.buildDockerfile(),
                size: size,
                os: config.os,
                architecture: config.architecture,
                workflowName: this.workflowName,
                workflowUrl: this.workflowUrl,
                logsUrl: this.logsUrl,
            }
        };
        console.log('binaryMutation payload:', imageBinaryVars.imageBinary);
        const binaryResult = await this.codefreshAPI.createImageBinary(imageBinaryVars);
        console.log(JSON.stringify(binaryResult, null, 2));
        const imageLayersVars = {
            imageLayers: {
                image: manifest.config.digest,
                layers: this.registryClient.getLayersFromHistory(config.history, manifest.layers),
            }
        };
        console.log('ImageLayers Mutation payload:', JSON.stringify(imageLayersVars.imageLayers, null, 2));
        const imageLayersResult = await this.codefreshAPI.createImageLayers(imageLayersVars)
            .catch((layersError) => {
            console.error(layersError);
        });
        console.log(JSON.stringify(imageLayersResult, null, 2));
        const imageRegistryVars = {
            imageRegistry: {
                binaryId: binaryResult.createImageBinary.id,
                imageName: this.image,
                repoDigest: manifest.config.repoDigest,
                created: configCreated,
                architecture: config.architecture,
                parentRepoDigest,
            }
        };
        console.log('registryMutation payload:', imageRegistryVars.imageRegistry);
        const registryResult = await this.codefreshAPI.createImageRegistry(imageRegistryVars);
        console.log(JSON.stringify(registryResult, null, 2));
        return { imageBinary: binaryResult.createImageBinary };
    }
}
exports.ImageInfoReporter = ImageInfoReporter;
async function reportImageInfo(inputs, notify) {
    const imageInfoReporter = new ImageInfoReporter(inputs, notify);
    await imageInfoReporter.reportImageInfo();
}
exports.reportImageInfo = reportImageInfo;
//# sourceMappingURL=index.js.map