import Promise from 'bluebird'
import _ from 'lodash'
import chalk from 'chalk'

import { GitEnricherInputs, Platform } from './types'
import { GitProviders, providers } from '../api/git-providers'
import { CodefreshAPI } from '../api/codefresh'
import { GerritService } from '../api/git-providers/gerrit'

async function resolvePullRequests(inputs: GitEnricherInputs, provider) {
  if (inputs.eventFilePR) {
    // for data got from file we need to expand prs with commits info
    return Promise.mapSeries(inputs.eventFilePR, async (pr) => {
      const commitsInfo = await provider.getCommitsInfo(inputs.repo, pr.number)
      return {
        ...pr,
        ...commitsInfo,
      }
    })
  }
  if (inputs.provider === providers.GERRIT) {
    return provider.getPullRequestsWithCommits(inputs.repo, inputs.gerritChangeId)
  }
  return provider.getPullRequestsWithCommits(inputs.repo, inputs.branch)
}

async function resolveBranch(inputs: GitEnricherInputs, provider) {
  if (inputs.provider === providers.GERRIT) {
    return provider.getBranch(inputs.repo, inputs.gerritChangeId)
  }
  return provider.getBranch(inputs.repo, inputs.branch, inputs.revision)
}

async function patchImageWithGitBranchData(codefreshAPI, provider, pullRequests, inputs) {
  if (_.isEmpty(pullRequests)) {
    console.warn(`no PRs have been found, looking for the branch data`)

    const branch = await resolveBranch(inputs, provider)
    if (branch) {
      // branch exists without PR, try to take git data from branch
      await codefreshAPI.patchImageWithGitBranchData(inputs.imageName, branch, inputs.provider)
      console.log(`image patched`)
    } else {
      console.warn(`branch "${inputs.branch}" not found, ensure that it is not deleted`)
    }
  } else {
    console.log(chalk.green(`retrieved prs ${JSON.stringify(pullRequests, null, 2)}`))

    // PR exists, taking git data from PR commits
    const lastCommit = GitProviders.getTheMostRecentCommit(pullRequests, inputs.revision)
    await codefreshAPI.patchImageWithGitBranchData(inputs.imageName, {
      name: inputs.provider === providers.GERRIT ? pullRequests[0].targetBranch : inputs.branch,
      commit: lastCommit.sha,
      commitMsg: lastCommit.message,
      commitURL: lastCommit.url,
      committerUsername: lastCommit.userName
    }, inputs.provider)
    console.log(`image patched`)
  }
}

export async function imageEnricherGitInfo(inputs: GitEnricherInputs) {
  console.log('starting git enricher library')

  const { cfHost, cfApiKey } = inputs
  const codefreshAPI = new CodefreshAPI(cfHost, cfApiKey)

  const Provider = GitProviders.get(inputs.provider)
  const provider = new Provider(inputs)

  if (inputs.provider === providers.GERRIT) {
      inputs.gerritChangeId = GerritService.parseChangeId(inputs.gerritChangeId)
  }

  const pullRequests = await resolvePullRequests(inputs, provider).catch((error) => {
      throw new Error((`Can't get pull requests. Error: ${error.message}`))
  })

  const gitopsImageReportingFeature = await codefreshAPI.isGitopsReportingEnabled(inputs.platform)

  if (inputs.platform === Platform.GITOPS || gitopsImageReportingFeature) {
    try {
      await patchImageWithGitBranchData(codefreshAPI, provider, pullRequests, inputs)
    } catch (error) {
      console.log(`Unable to enrich GitOps image ${inputs.imageName} with branch data`)
      if (inputs.platform === Platform.GITOPS) {
        throw error
      }
    }

  }

  if (_.isEmpty(pullRequests)) {
    console.warn(`pullRequests array is empty`)
    return
  }

  // Report prs as annotations
  for (const pr of pullRequests) {
    if (inputs.platform === Platform.CLASSIC) {
      try {
        console.log(`creating classic annotation for image ${inputs.imageName}`)
        await codefreshAPI.createPullRequestAnnotationClassic(inputs.imageName, pr)
        console.log(`classic annotation created`)
      } catch (error) {
        throw new Error(`Failed to assign pull request ${pr.number} to your classic image ${inputs.imageName}, reason: ${error.message}`)
      }
    }

    if (inputs.platform === Platform.GITOPS || gitopsImageReportingFeature) {
      try {
        console.log(`creating gitops annotation for image ${inputs.imageName}`)
        const annotationKey = `#${inputs.provider}#${inputs.repo}#${pr.number}`
        await codefreshAPI.createPullRequestAnnotation(annotationKey, inputs.imageName, pr)
        console.log(`gitops annotation created`)
      } catch (error) {
        throw new Error(`Failed to assign pull request ${pr.number} to your gitops image ${inputs.imageName}, reason: ${error.message}`)
      }
    }
  }
}
