import _ from 'lodash'

import { RegistryClient } from '../api/registries'
import { CodefreshAPI } from '../api/codefresh'
import { ReportImageInputs, NOTIFICATIONS, NotifierFunction } from './types'

export class ImageInfoReporter {
  notify: NotifierFunction
  image: string
  registryClient: RegistryClient
  registry: any
  codefreshAPI: CodefreshAPI
  apiKey: string
  host?: string
  workflowName?: string
  workflowUrl?: string
  logsUrl?: string

  constructor(inputs: ReportImageInputs, notify: NotifierFunction) {
    this.image = inputs.imageName
    this.registryClient = new RegistryClient(inputs)
    this.workflowName = inputs.workflow?.name
    this.workflowUrl = inputs.workflow?.workflowUrl
    this.logsUrl = inputs.workflow?.logsUrl
    this.host = inputs.codefresh.host
    this.apiKey = inputs.codefresh.apiKey
    this.notify = notify
    this.codefreshAPI = new CodefreshAPI(this.host, this.apiKey)
  }

  async reportImageInfo() {
    console.log('starting image reporter library')
    const registry = await this.getRegistry()
    const manifest = await registry.getManifest()

    if (manifest.manifests) {
      // reporting multi arch image
      const parentRepoDigest = manifest.config?.repoDigest
      const reportResult = await Promise.all(_.map(manifest?.manifests, async (manifestConfig) => {
        if (manifestConfig.platform.os === 'unknown') {
          return {}
        }
        const archManifest = await registry.getManifest(manifestConfig.digest, manifestConfig.mediaType)
        console.log(`image manifest has been fetched:`, '\n', archManifest)
        return await this.getConfigAndReportImageInfo(archManifest, parentRepoDigest)
      }))
      const parentRegistryResult = await this.codefreshAPI.createImageRegistry({
        imageRegistry: {
          binaryId: reportResult[0].imageBinary.id,
          imageName: this.image,
          repoDigest: parentRepoDigest,
          created: new Date(),
          manifestList: true
        }
      })
      console.log('registryMutation payload:', parentRegistryResult)
    } else {
      console.log(`image manifest has been fetched:`, '\n', manifest)
      await this.getConfigAndReportImageInfo(manifest)
    }

    console.log('image reported successfully')
  }

  private async getRegistry() {
    if (this.registry) {
      return this.registry
    }
    const client = await this.registryClient.getRegistryClient(this.image)
    console.log(`using ${client.constructor.name} client`)

    const registry = client.repoTag(this.image)
    this.registry = registry
    return registry
  }

  private async getConfigAndReportImageInfo(manifest: any, parentRepoDigest?: string)  {

    const registry = await this.getRegistry()
    const config = await registry.getConfig(manifest)

    console.log(`image config has been fetched:`, '\n', config)

    const repositoryName = _.get(this.registryClient.parseImageName(this.image), 'repository')
    const imageLink = `${this.host}/2.0/images/${encodeURIComponent(repositoryName)}/${manifest.config.digest}/${encodeURIComponent(this.image)}`

    const size = this.registryClient.getRegistryImageSize(manifest)


    /***
     * Notification
     ***/
    try {
      await this.notify(NOTIFICATIONS.IMAGE_NAME, this.image)
    } catch (error) {
      console.error(`Can't call notification "${NOTIFICATIONS.IMAGE_NAME}". ${error.message}`)
    }
    try {
      await this.notify(NOTIFICATIONS.IMAGE_SHA, _.get(manifest, 'config.digest'))
    } catch (error) {
      console.error(`Can't call notification "${NOTIFICATIONS.IMAGE_SHA}". ${error.message}`)
    }
    try {
      await this.notify(NOTIFICATIONS.IMAGE_LINK, imageLink)
    } catch (error) {
      console.error(`Can't call notification "${NOTIFICATIONS.IMAGE_LINK}". ${error.message}`)
    }


    const configCreated = config.created || new Date().toISOString()

    /***
     * Image Binary
     ***/
    const imageBinaryVars = {
      imageBinary: {
        id: manifest.config.digest,
        created: configCreated,
        imageName: this.image,
        dockerFile: await this.registryClient.buildDockerfile(),
        size: size,
        os: config.os,
        architecture: config.architecture,
        workflowName: this.workflowName,
        workflowUrl: this.workflowUrl,
        logsUrl: this.logsUrl,
      }
    }

    console.log('binaryMutation payload:', imageBinaryVars.imageBinary)

    const binaryResult = await this.codefreshAPI.createImageBinary(imageBinaryVars)

    console.log(JSON.stringify(binaryResult, null, 2))

    /***
     * Image Layers
     ***/
    const imageLayersVars = {
      imageLayers: {
        image: manifest.config.digest,
        layers: this.registryClient.getLayersFromHistory(config.history, manifest.layers),
      }
    }

    console.log('ImageLayers Mutation payload:', JSON.stringify(imageLayersVars.imageLayers, null, 2))

    const imageLayersResult = await this.codefreshAPI.createImageLayers(imageLayersVars)
        .catch((layersError) => {
          console.error(layersError)
        })

    console.log(JSON.stringify(imageLayersResult, null, 2))

    /***
     * Image Registry
     ***/
    const imageRegistryVars = {
      imageRegistry: {
        binaryId: binaryResult.createImageBinary.id,
        imageName: this.image,
        repoDigest: manifest.config.repoDigest,
        created: configCreated,
        architecture: config.architecture,
        parentRepoDigest,
      }
    }

    console.log('registryMutation payload:', imageRegistryVars.imageRegistry)

    const registryResult = await this.codefreshAPI.createImageRegistry(imageRegistryVars)

    console.log(JSON.stringify(registryResult, null, 2))

    return { imageBinary: binaryResult.createImageBinary }
  }
}

export async function reportImageInfo(inputs: ReportImageInputs, notify: NotifierFunction) {
  const imageInfoReporter = new ImageInfoReporter(inputs, notify)
  await imageInfoReporter.reportImageInfo()
}
